'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babylon = require('babylon');

var _s2sUtils = require('s2s-utils');

var _flowFaker = require('flow-faker');

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _json = require('json5');

var _json2 = _interopRequireDefault(_json);

var _typedAssign = require('typed-assign');

var _typedAssign2 = _interopRequireDefault(_typedAssign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'

const INITIAL_STATE = 'initialState';

const parseFromObj = obj => (0, _babylon.parseExpression)(JSON.stringify(obj), {
  plugins: ['flow']
});

const getObj = n => _json2.default.parse((0, _babelGenerator2.default)(n).code);

exports.default = () => {
  return {
    inherits: _s2sUtils.inheritsOpts,
    name: 's2s-initial-state-creater',
    visitor: {
      Program(programPath, state) {
        const { opts: { from } } = state;
        programPath.traverse({
          VariableDeclarator(nodePath) {
            const idPath = nodePath.get('id');
            if (idPath.node.name !== INITIAL_STATE) {
              return;
            }

            const { line, column } = nodePath.get('loc.start').node;
            const flowInfoObj = (0, _flowFaker.flowFakerSync)(from, {
              row: line,
              column: column + 1
            });

            const NodeFromFlowtype = parseFromObj(flowInfoObj);
            const NodeFromVariable = nodePath.get('init').node;

            const mergedObj = (0, _typedAssign2.default)({}, getObj(NodeFromVariable), getObj(NodeFromFlowtype));

            nodePath.get('init').replaceWith(parseFromObj(mergedObj));
          }
        });
      }
    }
  };
};